# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/aws.ipynb.

# %% auto 0
__all__ = ['AwsClient']

# %% ../nbs/aws.ipynb 3
import zipfile
import boto3
import tempfile
import shutil
import os

# %% ../nbs/aws.ipynb 4
class AwsClient:
    """
    A client for interacting with AWS services, specifically for operations related to Amazon S3.
    """

    def __init__(self, aws_access_key_id: str, aws_secret_access_key: str, region_name: str):
        """
        Initializes the client with AWS credentials and region information.

        Args:
            aws_access_key_id (str): AWS access key ID.
            aws_secret_access_key (str): AWS secret access key.
            region_name (str): AWS region name.
        """
        self.s3 = boto3.client(
            's3',
            aws_access_key_id=aws_access_key_id,
            aws_secret_access_key=aws_secret_access_key,
            region_name=region_name
        )

    def upload_to_s3(self, file_path: str, bucket_name: str, object_name: str):
        """
        Uploads a file to an Amazon S3 bucket.

        Args:
            file_path (str): Local path to the file.
            bucket_name (str): Name of the S3 bucket.
            object_name (str): Object name in the S3 bucket (i.e., S3 key).
        """
        self.s3.upload_file(file_path, bucket_name, object_name)

    def extract_zip_with_correct_encoding(self, zip_file_path: str, extract_to_path: str, encoding: str = 'cp437'):
        """
        Extracts a ZIP file with specific encoding for the file names.

        Args:
            zip_file_path (str): Path to the ZIP file.
            extract_to_path (str): Path to extract the ZIP contents to.
            encoding (str, optional): Encoding to use for file names. Defaults to 'cp437'.
        """
        with zipfile.ZipFile(zip_file_path, 'r') as zip_ref:
            for member in zip_ref.infolist():
                # ファイル名のエンコーディングを修正
                file_name = member.filename.encode('cp437').decode(encoding)
                target_path = os.path.join(extract_to_path, file_name)
                
                # ディレクトリの場合は作成
                if member.is_dir():
                    os.makedirs(target_path, exist_ok=True)
                else:
                    # ファイルを展開
                    with zip_ref.open(member, 'r') as source, open(target_path, 'wb') as target:
                        shutil.copyfileobj(source, target)

    def upload_to_s3_zip(self, zip_file_path: str, dst_path: str, bucket_name: str):
        """
        Extracts a ZIP file and uploads its contents to an Amazon S3 bucket.

        Args:
            zip_file_path (str): Path to the ZIP file.
            dst_path (str): Destination path in the S3 bucket.
            bucket_name (str): Name of the S3 bucket.
        """

        with tempfile.TemporaryDirectory() as tmpdirname:
            # Extract the ZIP file
            with zipfile.ZipFile(zip_file_path, 'r') as zip_ref:
                zip_ref.extractall(tmpdirname)

            # Upload the extracted files to S3
            for root, dirs, files in os.walk(tmpdirname):
                for file in files:
                    local_path = os.path.join(root, file)

                    s3_path = os.path.join(dst_path, os.path.relpath(local_path, tmpdirname)).replace("\\", "/")

                    # Perform the actual upload
                    self.s3.upload_file(local_path, bucket_name, s3_path)



# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/task.ipynb.

# %% auto 0
__all__ = ['Task']

# %% ../nbs/task.ipynb 3
from .api import ArchivematicaAPIClient
from .aws import AwsClient
from typing import Optional

# %% ../nbs/task.ipynb 4
class Task:

    def __init__(self, dashboard_url: str, dashboard_username: str, dashboard_api_key: str,
                 storage_service_url: str, storage_service_username: str, storage_service_password: str,
                 aws_access_key_id: str, aws_secret_access_key: str):
        """
        Task クラスのコンストラクタ。Matica APIとAWS S3クライアントの初期化を行う。

        Args:
            dashboard_url (str): MaticaダッシュボードのURL。
            dashboard_username (str): Maticaダッシュボードのユーザー名。
            dashboard_api_key (str): MaticaダッシュボードのAPIキー。
            storage_service_url (str): ストレージサービスのURL。
            storage_service_username (str): ストレージサービスのユーザー名。
            storage_service_password (str): ストレージサービスのパスワード。
            aws_access_key_id (str): AWSアクセスキーID。
            aws_secret_access_key (str): AWSシークレットアクセスキー。
        """
        self.matica_client = ArchivematicaAPIClient(dashboard_url, dashboard_username, dashboard_api_key,
                                             storage_service_url, storage_service_username, storage_service_password)
        self.aws_client = AwsClient(aws_access_key_id, aws_secret_access_key, "us-east-1")

    @staticmethod
    def ensure_slash_suffix(s: str) -> str:
        """指定された文字列の末尾にスラッシュがなければ追加する

        Args:
            s (str): 処理する文字列

        Returns:
            str: 末尾にスラッシュが保証された文字列
        """
        return s if s.endswith('/') else f'{s}/'

    @staticmethod
    def main(dashboard_url: str, dashboard_username: str, dashboard_api_key: str,
             storage_service_url: str, storage_service_username: str, storage_service_password: str,
             aws_access_key_id: str, aws_secret_access_key: str, task_id: str, transfer_type: str, 
             transfer_name: str, file_path: str, location_uuid: str, processing_config: str, 
             bucket_name: str, transfer_source_prefix: str) -> Optional[str]:
        """
        メインの処理を実行する静的メソッド。ZIPファイルをS3にアップロードし、Maticaでの処理をトリガーする。

        Args:
            以下の引数は __init__ メソッドと同様。
            task_id (str): タスクID。
            transfer_type (str): 転送タイプ。
            transfer_name (str): 転送名。
            file_path (str): ファイルパス。
            location_uuid (str): 位置UUID。
            processing_config (str): 処理設定。
            bucket_name (str): バケット名。
            transfer_source_prefix (str): 転送ソースプレフィックス。

        Returns:
            Optional[str]: 成功時には処理されたファイルのURLを返し、失敗時にはNoneを返す。
        """
        
        task = Task(dashboard_url, 
        dashboard_username,
        dashboard_api_key,
        storage_service_url,
        storage_service_username,
        storage_service_password,
        aws_access_key_id,
        aws_secret_access_key)

        matica_client = task.matica_client

        transfer_source_prefix = Task.ensure_slash_suffix(transfer_source_prefix)
        s3_path = f"{transfer_source_prefix}{task_id}"

        # bucket_name = "archivematica.aws.ldas.jp"
        # bucket_name

        task.aws_client.upload_to_s3_zip(file_path, s3_path, bucket_name)

        # location_uuid = "8ba875f0-0e78-4c98-a608-751a362e3194"
        # processing_config = "automated"

        transfer_UUID = task.matica_client.v2beta_package(transfer_type, "", location_uuid, "/" + s3_path, transfer_name, processing_config)

        sip_UUID = matica_client.check_transfer_status(transfer_UUID)
        ingest_UUID = matica_client.ingest(sip_UUID)

        url = "https://s3.amazonaws.com/" + bucket_name + matica_client.get_current_full_path(ingest_UUID)

        return url



# Autogenerated by nbdev

d = { 'settings': { 'branch': 'main',
                'doc_baseurl': '/archivematica_tools',
                'doc_host': 'https://nakamura196.github.io',
                'git_url': 'https://github.com/nakamura196/archivematica_tools',
                'lib_path': 'archivematica_tools'},
  'syms': { 'archivematica_tools.api': { 'archivematica_tools.api.ArchivematicaAPIClient': ( 'api.html#archivematicaapiclient',
                                                                                             'archivematica_tools/api.py'),
                                         'archivematica_tools.api.ArchivematicaAPIClient.__init__': ( 'api.html#archivematicaapiclient.__init__',
                                                                                                      'archivematica_tools/api.py'),
                                         'archivematica_tools.api.ArchivematicaAPIClient.approve_transfer': ( 'api.html#archivematicaapiclient.approve_transfer',
                                                                                                              'archivematica_tools/api.py'),
                                         'archivematica_tools.api.ArchivematicaAPIClient.check_transfer_status': ( 'api.html#archivematicaapiclient.check_transfer_status',
                                                                                                                   'archivematica_tools/api.py'),
                                         'archivematica_tools.api.ArchivematicaAPIClient.get_aip_url': ( 'api.html#archivematicaapiclient.get_aip_url',
                                                                                                         'archivematica_tools/api.py'),
                                         'archivematica_tools.api.ArchivematicaAPIClient.get_current_full_path': ( 'api.html#archivematicaapiclient.get_current_full_path',
                                                                                                                   'archivematica_tools/api.py'),
                                         'archivematica_tools.api.ArchivematicaAPIClient.ingest': ( 'api.html#archivematicaapiclient.ingest',
                                                                                                    'archivematica_tools/api.py'),
                                         'archivematica_tools.api.ArchivematicaAPIClient.v2beta_package': ( 'api.html#archivematicaapiclient.v2beta_package',
                                                                                                            'archivematica_tools/api.py')},
            'archivematica_tools.aws': { 'archivematica_tools.aws.AwsClient': ('aws.html#awsclient', 'archivematica_tools/aws.py'),
                                         'archivematica_tools.aws.AwsClient.__init__': ( 'aws.html#awsclient.__init__',
                                                                                         'archivematica_tools/aws.py'),
                                         'archivematica_tools.aws.AwsClient.extract_zip_with_correct_encoding': ( 'aws.html#awsclient.extract_zip_with_correct_encoding',
                                                                                                                  'archivematica_tools/aws.py'),
                                         'archivematica_tools.aws.AwsClient.upload_to_s3': ( 'aws.html#awsclient.upload_to_s3',
                                                                                             'archivematica_tools/aws.py'),
                                         'archivematica_tools.aws.AwsClient.upload_to_s3_zip': ( 'aws.html#awsclient.upload_to_s3_zip',
                                                                                                 'archivematica_tools/aws.py')},
            'archivematica_tools.task': { 'archivematica_tools.task.Task': ('task.html#task', 'archivematica_tools/task.py'),
                                          'archivematica_tools.task.Task.__init__': ( 'task.html#task.__init__',
                                                                                      'archivematica_tools/task.py'),
                                          'archivematica_tools.task.Task.ensure_slash_suffix': ( 'task.html#task.ensure_slash_suffix',
                                                                                                 'archivematica_tools/task.py'),
                                          'archivematica_tools.task.Task.main': ('task.html#task.main', 'archivematica_tools/task.py')}}}



# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/api.ipynb.

# %% auto 0
__all__ = ['ArchivematicaAPIClient']

# %% ../nbs/api.ipynb 3
import base64
import requests
import time
from requests.auth import HTTPBasicAuth
from typing import Optional

# %% ../nbs/api.ipynb 4
class ArchivematicaAPIClient(object):
    """
    A client for interacting with Matica's API for file uploading, transferring, and ingesting processes.
    """

    TIME_SPAN = 2  # Time to wait in loops, in seconds

    def __init__(self, dashboard_url: str, dashboard_username: str, dashboard_api_key: str, storage_service_url: str, storage_service_username: str, storage_service_password: str):
        """
        Initializes the ArchivematicaAPIClient with necessary details for API interactions.

        Args:
            dashboard_url (str): URL of the dashboard.
            dashboard_username (str): Username for the dashboard.
            dashboard_api_key (str): API key for the dashboard.
            storage_service_url (str): URL of the storage service.
            storage_service_username (str): Username for the storage service.
            storage_service_password (str): Password for the storage service.
        """

        self.dashbord_url = dashboard_url
        self.dashboard_username = dashboard_username
        self.dashboard_api_key = dashboard_api_key

        self.dash_board_headers = {
            'Authorization': f'ApiKey {dashboard_username}:{dashboard_api_key}',
            "Content-Type": "application/json"
        }

        self.dash_board_endpoint = f"{dashboard_url}/api"

        self.storage_service_url = storage_service_url
        self.storage_username = storage_service_username
        self.storage_password = storage_service_password
    
    def v2beta_package(self, transfer_type: str, transfer_accession: str, location_uuid: str, path: str, name: str, processing_config: str) -> str:
        """
        Initiates a file transfer process.

        Args:
            transfer_type: The type of the transfer.
            transfer_accession: The accession number for the transfer.

        Returns:
            The directory associated with the transfer.
        """

        path = f"{location_uuid}:{path}"

        path_encoded = base64.b64encode(path.encode()).decode()
        data = {
            "name": name,
            "type": transfer_type,
            "accession": transfer_accession,
            "processing_config": processing_config,
            "path": path_encoded,
        }

        response = requests.post(f'{self.dashbord_url}/api/v2beta/package/', headers=self.dash_board_headers, json=data)

        self.transfer_UUID = response.json()["id"]
        return self.transfer_UUID
    
    def approve_transfer(self, transfer_type: str, directory: str) -> str:
        """
        Approves a file transfer after initiation.

        Args:
            transfer_type (str): The type of the transfer.
            directory (str): The directory associated with the transfer.

        Returns:
            Optional[str]: The UUID of the approved transfer, if available.
        """
        while True:
            data = {"type": transfer_type, "directory": directory}
            response = requests.post(f'{self.dashbord_url}/api/transfer/approve/', headers=self.dash_board_headers, data=data)
            if "uuid" in response.json():
                self.transfer_UUID = response.json()["uuid"]
                break
            time.sleep(self.TIME_SPAN)
        return self.transfer_UUID

    def check_transfer_status(self, transfer_UUID: str) -> str:
        """
        Checks the status of a file transfer until it is no longer processing.

        Args:
            transfer_UUID: The UUID of the transfer to check.

        Returns:
            The SIP UUID of the transfer.
        """
        while True:
            response = requests.get(f'{self.dashbord_url}/api/transfer/status/{transfer_UUID}', headers=self.dash_board_headers)
            r = response.json()
            if "status" in r and r["status"] != "PROCESSING":
                self.sip_uuid = r["sip_uuid"]
                break
            time.sleep(self.TIME_SPAN)
        return self.sip_uuid

    def ingest(self, sip_UUID: str) -> str:
        """
        Waits for the ingestion process to complete and returns the UUID of the ingested AIP.

        Args:
            sip_UUID: The SIP UUID to ingest.

        Returns:
            The UUID of the ingested AIP.
        """
        while True:
            response = requests.get(f'{self.dash_board_endpoint}/ingest/status/{sip_UUID}', headers=self.dash_board_headers)
            r = response.json()
            if "status" in r and r["status"] != "PROCESSING":
                return r["uuid"]
            time.sleep(self.TIME_SPAN)

    def get_aip_url(self, ingest_UUID: str) -> str:
        """
        Constructs and returns the URL for downloading the ingested AIP.

        Args:
            ingest_UUID: The UUID of the ingested AIP.

        Returns:
            The URL for downloading the AIP.
        """
        endpoint = self.storage_service_url + "/api"
        url = f"{endpoint}/v2/file/{ingest_UUID}/download/"
        return url
    
    def get_current_full_path(self, ingest_UUID: str) -> Optional[str]:
        """
        Retrieves the current full path of the ingested AIP by making an API call.

        Args:
            ingest_UUID (str): The UUID of the ingested AIP.

        Returns:
            Optional[str]: The current full path of the AIP if the request is successful, None otherwise.
        """
        endpoint = self.storage_service_url + "/api"
        url = f"{endpoint}/v2/file/{ingest_UUID}/"

        try:
            response = requests.get(url, auth=HTTPBasicAuth(self.storage_username, self.storage_password))
            response.raise_for_status()  # Raises an exception for 4XX or 5XX errors
            return response.json().get("current_full_path")
        except requests.RequestException as e:
            print(f"Error retrieving current full path: {e}")
            return None

